#define PIC16F676

#define LCD

#include "16f676.h"
#pragma config |= 0x318C // INT OSC; 318C with Watch Dog;

#pragma update_RP 1

typedef unsigned char byte;
typedef unsigned long int LONG;
typedef bit BOOL;

//--- PORT DEFINITIONS ---------------------------------------------------
//               76543210
#define  TrisA 0b00011000                
//               ||||||||
//               |||||||
//               ||||||
//               |||||SERVO
//               ||||BUTTON
//               |||CH3
//               ||LED
//               |---
//               ---

//               76543210
#define  TrisC 0b00000000
//               ||||||||
//               |||||||D4
//               ||||||D5
//               |||||D6 
//               ||||D7
//               |||E 
//               ||A0
//               |---
//               ---

// LCD connector;
// 1   2   3   4   5   6   7   8   9   10
// GND U+  A0  E   D4  D5  D6  D7  UBL+

#pragma  bit ACCELX   @ PORTA.0   //        - PIC-13
#pragma  bit ACCELY   @ PORTA.1   //        - PIC-12
#pragma  bit SERVO    @ PORTA.2   //        - PIC-11
#pragma  bit BUTTON   @ PORTA.3   //        - PIC-4 
#pragma  bit CH3      @ PORTA.4   //        - PIC-3 
#pragma  bit LED      @ PORTA.5   //        - PIC-2 

#pragma  bit LCD_A0   @ PORTC.5   // LCD-4  - PIC-5 
#pragma  bit LCD_E    @ PORTC.4   // LCD-6  - PIC-6 
#pragma  bit LCD_D4   @ PORTC.0   // LCD-11 - PIC-10
#pragma  bit LCD_D5   @ PORTC.1   // LCD-12 - PIC-9
#pragma  bit LCD_D6   @ PORTC.2   // LCD-13 - PIC-8
#pragma  bit LCD_D7   @ PORTC.3   // LCD-14 - PIC-7

#define BUTTON_PRESSED 1
//------------------------------------------------------------------------

#include "int16CXX.H"

#define SET_RP0 
//#asm \
//BSF   0x03,RP0 \
//#endasm

#define CLR_RP0 
//#asm \
//BCF   0x03,RP0 \
//#endasm

LONG Servo;
bit  ServoStart;
byte prevWidth;

#define SERVO_PERIOD	20000
#define SERVO_NEUTRAL	1500
#define SERVO_SPAN	500
#define SERVO_HIGH	100
#define SERVO_MAX (SERVO_NEUTRAL+SERVO_SPAN)
#define SERVO_START	500

#pragma origin 4
interrupt serverX( void)
{
/* W and STATUS are saved by the next macro. PCLATH is also
   saved if necessary. The code produced is CPU-dependent. */

int_save_registers    // W, STATUS (and PCLATH if required)
//char sv_FSR = FSR;  // save FSR if required
char sv_PCLATH = PCLATH; // save PCLATH

  // handle the interrupt (insert application code here)

   CLR_RP0;
   T1CON = 0b00000000; // Stop timer;

   LONG width;
   if (ServoStart == 0) {
     SERVO = 1; 
     ServoStart = 1;
     width = Servo;
     prevWidth = width;
   } else {
     width = SERVO_PERIOD-prevWidth;
     SERVO = 0;
     ServoStart = 0;
   }
   if (width == 0)
     width.low8 = 1;
   width = -width;

   TMR1H = width.high8;
   TMR1L = width.low8;

   T1CON = 0b00000001; // Resume counting;
   TMR1IF = 0;         

PCLATH = sv_PCLATH;    // restore PCLATH
//FSR = sv_FSR;       // restore FSR if saved
int_restore_registers // W, STATUS (and PCLATH if required)
}

//#include "BIN\MATH16.H"

//--- PAUSE MAKING FUNCTIONS ---------------------------------------------
void Delay100us (byte aTs) // pause for aTs*100 us;
{ // 4+x*3 us = 4+3*32=100 us;
 do  {
     char i = 33;            /* 2 IS */
     do ; while ( --i > 0);  /* 26 * 3 - 1 = 77 IS */
 } while ( --aTs > 0);         /* 3 IS */
}

void Delay10ms (byte aTs) // pause for aTs*10 ms;
{
 for (; aTs > 0; aTs--) {
   Delay100us (100);
   clrwdt();
 }
}

/*
void DelayMs (unsigned char aTs) // pause for aTs*1 ms;
{
 for (; aTs > 0; aTs--) {
   clrwdt();
   Delay100us (10);
 }
}
*/

#include "lcd.c"

void InitServo() {
  // init channels;
  Servo = SERVO_NEUTRAL;
}

//--- ANALOG SETUP DATA --------------------------------------------------
#define ADChannel_CH3		3

LONG CH3Voltage;

#define NUM_VOLTAGE_SAMPLES     8  // 2, 4, 8, 16

LONG ReadADC(byte piADChannel) {
  CLR_RP0;
  byte a = (piADChannel << 2);
  ADCON0 = (ADCON0 & 0b111.000.11) | a;
  Delay100us(10);
  GO = 1;
  while (GO) { };

  long unsigned int u;
  u.high8 = ADRESH;
  SET_RP0;
  u.low8 = ADRESL;
  CLR_RP0;
  return u;
}

void ReadChannels() {
  CH3Voltage = 0;
  signed int i = NUM_VOLTAGE_SAMPLES;
  while (i > 0) {
    CH3Voltage += ReadADC(ADChannel_CH3);
    i--;
  }

  CH3Voltage /= NUM_VOLTAGE_SAMPLES; // 1:1
}

// Find if channels Cell1, Cell2, Cell3 and Temp are available;
void ReadAndOutputAvailableChannels() {
  ReadChannels();

  signed long x = (((signed long) CH3Voltage)-1023/2);
  x*=2;
  x+= SERVO_NEUTRAL;
  if (x < 0) x=0;

  GIE = 0;
  Servo = (LONG) x;
  GIE = 1;

#ifdef LCD
  LCD_CURSOR_LINE_1 (0);  WriteNum (CH3Voltage);
  LCD_CURSOR_LINE_2 (0);  WriteNum (Servo);
#endif
}

void Beep(byte Beep10ms, byte Pause10ms) {
  LED=1;
  Delay10ms(Beep10ms);
  LED=0;
  Delay10ms(Pause10ms);
}

char GetOsccal();							// Defined at end of file

void main() {
  W = GetOsccal();						// Set built-in oscillator calibration
  SET_RP0;
  OSCCAL = W;
  
  TRISA = TrisA;
  TRISC = TrisC;
  OPTION = 0b11001110; // no pull-up, int/b0 on 0->1; Prescalder 64 -> ~1 sec. to WDT;
  CLR_RP0;
  PORTA = 0; 
  PORTC = 0;

  CMCON = 0b.0000.0111; // comparator is turned off;

  SET_RP0;
  ANSEL = (1 << ADChannel_CH3);

  ADCON1 = 0b0.101.0000;      //  Select the Clock as Fosc/16

  ADIE = 0;

  CLR_RP0;
  ADCON0 = 0b10.0.000.01;     //  Turn on the ADC
                              //   Bit 7 - Right Justified Sample
                              //   Bit 6 - Dont Use VRef
                              //   Bit 4:2 - Channel 0
                              //   Bit 1 - Do not Start
                              //   Bit 0 - Turn on ADC

#ifdef LCD
  LCD_Init();          
  lcdWriteln ("Test");
  Delay10ms (100);
  LCD_CLEAR;
#endif

  InitServo();
  ServoStart = 0;
  SERVO = 0;

  // Init Timer1;
  TMR1L = 0xFF;
  TMR1H = 0;
  T1CON = 0b00000001; // Resume counting;
  TMR1IF = 0;           

  SET_RP0;
  PIE1 = 1; 

  PEIE = 1;
  GIE = 1;

  while (1) {
    clrwdt(); 
    ReadAndOutputAvailableChannels();

  };
}

#pragma origin 0x3FF
// This is a dummy - actual procedure is preset in chip and will override
// A change of 1 causes about 1% change in frequency
char GetOsccal() {return 0x0;}

/*
// Initialize EEPROM data
#define EEPROM_START 0x2100
#pragma cdata[EEPROM_START]
#pragma cdata[] = MAGIC, 0x00	
*/
